@@ -1,7 +1,4 @@
-/* http.c - jonclegg@yahoo.com
- * library routines for handling shoutcast centric http parsing
- *
- * This program is free software; you can redistribute it and/or modify
+/* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
@@ -19,17 +16,86 @@
 #include <string.h>
 #include <stdlib.h>
 #include "compat.h"
+#include "srtypes.h"
+#include "socklib.h"
 #include "http.h"
 #include "util.h"
 #include "debug.h"
 
-/*********************************************************************************
- * Private functions
- *********************************************************************************/
+/******************************************************************************
+ * Function prototypes
+ *****************************************************************************/
 static char* make_auth_header(const char *header_name, 
 			      const char *username, const char *password);
-char* b64enc(const char *buf, int size);
+static error_code httplib_get_sc_header(const char* url, HSOCKET *sock, 
+					SR_HTTP_HEADER *info);
+static char* b64enc(const char *buf, int size);
 
+/******************************************************************************
+ * Private Vars
+ *****************************************************************************/
+#define MAX_PLS_LEN 8192
+#define MAX_M3U_LEN 8192
+
+
+/******************************************************************************
+ * Functions
+ *****************************************************************************/
+
+/* Connect to a shoutcast type stream, leaves when it's about to 
+   get the header info */
+error_code
+httplib_sc_connect (HSOCKET *sock, const char *url, const char *proxyurl, 
+		 SR_HTTP_HEADER *info, char *useragent, char *if_name)
+{
+    char headbuf[MAX_HEADER_LEN];
+    URLINFO url_info;
+    int ret;
+
+    while (1) {
+	debug_printf ("***** URL = %s *****\n", url);
+	debug_printf("inet_sc_connect(): calling httplib_parse_url\n");
+	if (proxyurl) {
+	    if ((ret = httplib_parse_url(proxyurl, &url_info)) != SR_SUCCESS) {
+		return ret;
+	    }
+	} else if ((ret = httplib_parse_url(url, &url_info)) != SR_SUCCESS) {
+	    return ret;
+	}
+
+	debug_printf("inet_sc_connect(): calling sockinit\n");
+	if ((ret = socklib_init()) != SR_SUCCESS)
+	    return ret;
+
+	debug_printf("inet_sc_connect(): calling sock_open: host=%s, port=%d\n",
+	    url_info.host, url_info.port);
+	if ((ret = socklib_open(sock, url_info.host, url_info.port, if_name)) != SR_SUCCESS)
+	    return ret;
+
+	debug_printf("inet_sc_connect(): calling httplib_construct_sc_request\n");
+	if ((ret = httplib_construct_sc_request(url, proxyurl, headbuf, useragent)) != SR_SUCCESS)
+	    return ret;
+
+	debug_printf("inet_sc_connect(): calling socklib_sendall\n");
+	if ((ret = socklib_sendall(sock, headbuf, strlen(headbuf))) < 0)
+	    return ret;
+
+	debug_printf("inet_sc_connect(): calling get_sc_header\n");
+	if ((ret = httplib_get_sc_header(url, sock, info)) != SR_SUCCESS)
+	    return ret;
+
+	if (*info->http_location) {
+	    /* RECURSIVE CASE */
+	    debug_printf ("Redirecting: %s\n", info->http_location);
+	    url = info->http_location;
+	    //inet_sc_connect(sock, info->http_location, proxyurl, info, useragent, if_name);
+	} else {
+	    break;
+	}
+    }
+
+    return SR_SUCCESS;
+}
 
 /*
  * Parse's a url as in http://host:port/path or host/path, etc..
@@ -292,6 +358,9 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
     else if (strstr(stempbr,"audio/aac")) {
 	info->content_type = CONTENT_TYPE_AAC;
     }
+    else if (strstr(stempbr,"audio/x-scpls")) {
+	info->content_type = CONTENT_TYPE_PLS;
+    }
     else {
 	info->content_type = CONTENT_TYPE_UNKNOWN;
     }
@@ -308,13 +377,13 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
 	    content_type_by_url = CONTENT_TYPE_MP3;
 	} else if (!strcmp (&url_info.path[url_path_len-4], ".nsv")) {
 	    content_type_by_url = CONTENT_TYPE_NSV;
+	} else if (!strcmp (&url_info.path[url_path_len-4], ".pls")) {
+	    content_type_by_url = CONTENT_TYPE_PLS;
+	} else if (!strcmp (&url_info.path[url_path_len-4], ".m3u")) {
+	    content_type_by_url = CONTENT_TYPE_M3U;
 	}
     }
 
-    //printf ("---------------\n");
-    //printf (header);
-    //printf ("---------------\n");
-
     // Try to guess the server
 
     // Check for Streamripper relay
@@ -408,6 +477,7 @@ httplib_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int
 {
     char *buf = (char *)malloc(size);
 
+#if defined (commentout)
     char* test_header = 
 	"HTTP/1.0 200 OK\r\n"
 	"Content-Type: application/ogg\r\n"
@@ -420,6 +490,7 @@ httplib_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int
 	"ice-public: 1\r\n"
 	"ice-url: http://www.oddsock.org\r\n"
 	"Server: Icecast 2.0.1\r\n\r\n";
+#endif
 
     if (!info || !header || size < 1)
 	return SR_ERROR_INVALID_PARAM;
@@ -432,8 +503,9 @@ httplib_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int
     return SR_SUCCESS;
 #endif
 
-    /* sprintf(buf, "HTTP/1.0 %d\r\n", info->icy_code); */
-    sprintf(buf, "HTTP/1.0 200 OK\r\n");
+    /* GCS: The code used to give HTTP instead of ICY for the response header, 
+	like this: sprintf(buf, "HTTP/1.0 200 OK\r\n"); */
+    sprintf (buf, "ICY 200 OK\r\n");
     strcat(header, buf);
 
     if (info->http_location[0])
@@ -516,12 +588,146 @@ httplib_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int
     return SR_SUCCESS;
 }
 
+/*
+[playlist]
+numberofentries=1
+File1=http://localhost:8000/
+Title1=(#1 - 530/18385) GCS hit radio
+Length1=-1
+Version=2
+*/
+error_code
+httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
+{
+    int s, bytes;
+    error_code rc;
+    char buf[MAX_PLS_LEN];
+    char location_buf[MAX_PLS_LEN];
+    char title_buf[MAX_PLS_LEN];
+    const int timeout = 30;
+
+    debug_printf ("Reading pls\n");
+    bytes = socklib_recvall (sock, buf, MAX_PLS_LEN, timeout);
+    if (bytes < SR_SUCCESS) return bytes;
+    if (bytes == 0 || bytes == MAX_PLS_LEN) {
+	debug_printf("Failed in getting PLS (%d bytes)\n", bytes);
+	return SR_ERROR_CANT_PARSE_PLS;
+    }
+    buf[bytes] = 0;
+
+    debug_printf ("Parsing pls\n");
+    debug_printf ("%s\n", buf);
+    debug_printf ("---\n");
+
+    rc = SR_ERROR_CANT_PARSE_PLS;
+    for (s = 1; s <= 99; s++) {
+	char buf1[20];
+	int num_scanned, used, total, open;
+	int best_open = 0;
+
+	sprintf (buf1, "File%d=", s);
+	if (!extract_header_value (buf, location_buf, buf1)) {
+	    break;
+	}
+	if (s == 1) {
+	    strcpy (info->http_location, location_buf);
+	    rc = SR_SUCCESS;
+	}
+	
+	sprintf (buf1, "Title%d=", s);
+	if (!extract_header_value (buf, title_buf, buf1)) {
+	    break;
+	}
+	num_scanned = sscanf (title_buf, "(#%*[0-9] - %d/%d",&used,&total);
+	if (num_scanned != 2) {
+	    break;
+	}
+	open = total - used;
+	if (open > best_open) {
+	    strcpy (info->http_location, location_buf);
+	    best_open = open;
+	}
+    }
+
+    strcpy (info->http_location, location_buf);
+
+    return rc;
+}
+
+/*
+#EXTM3U
+#EXTINF:111,3rd Bass - Al z A-B-Cee z
+mp3/3rd Bass/3rd bass - Al z A-B-Cee z.mp3
+*/
+error_code
+httplib_get_m3u (HSOCKET *sock, SR_HTTP_HEADER *info)
+{
+    int bytes;
+    char buf[MAX_M3U_LEN];
+    const int timeout = 30;
+    char* p;
+
+    debug_printf ("Reading m3u\n");
+    bytes = socklib_recvall (sock, buf, MAX_M3U_LEN, timeout);
+    if (bytes < SR_SUCCESS) return bytes;
+    if (bytes == 0 || bytes == MAX_M3U_LEN) {
+	debug_printf("Failed in getting M3U (%d bytes)\n", bytes);
+	return SR_ERROR_CANT_PARSE_M3U;
+    }
+    buf[bytes] = 0;
+
+    debug_printf ("Parsing m3u\n");
+    debug_printf ("%s\n", buf);
+    debug_printf ("---\n");
+
+    for (p = strtok (buf,"\n"); p!= 0; p = strtok (0, "\n")) {
+	size_t len;
+	if (p[0] == '#') {
+	    continue;
+	}
+	len = strlen (p);
+	if (len > 4 && !strcmp (&p[len-4], ".mp3")) {
+	    continue;
+	}
+	strcpy (info->http_location, p);
+	debug_printf ("Redirecting from M3U to: %s\n", p);
+	return SR_SUCCESS;
+    }
+
+    debug_printf ("Failed parsing M3U\n");
+    return SR_ERROR_CANT_PARSE_M3U;
+}
+
+static error_code
+httplib_get_sc_header(const char* url, HSOCKET *sock, SR_HTTP_HEADER *info)
+{
+    int ret;
+    char headbuf[MAX_HEADER_LEN] = {'\0'};
+
+    if ((ret = socklib_read_header(sock, headbuf, MAX_HEADER_LEN, NULL)) != SR_SUCCESS)
+	return ret;
+
+    if ((ret = httplib_parse_sc_header(url, headbuf, info)) != SR_SUCCESS)
+	return ret;
+
+    if (info->content_type == CONTENT_TYPE_PLS) {
+	ret = httplib_get_pls (sock, info);
+	if (ret != SR_SUCCESS) return ret;
+    } else if (info->content_type == CONTENT_TYPE_M3U) {
+	ret = httplib_get_m3u (sock, info);
+	if (ret != SR_SUCCESS) return ret;
+    }
+
+    return SR_SUCCESS;
+}
+
 // taken from:
 // Copyright (c) 2000 Virtual Unlimited B.V.
 // Author: Bob Deblier <bob@virtualunlimited.com>
 // thanks bob ;)
 #define CHARS_PER_LINE  72
-char* b64enc(const char *inbuf, int size)
+static char* 
+b64enc(const char *inbuf, int size)
 {
     /* encode 72 characters per line */
     static const char* to_b64 =
