@@ -1,7 +1,7 @@
 #ifndef __PARSE_H__
 #define __PARSE_H__
 
-#include "types.h"
+#include "srtypes.h"
 
 void init_metadata_parser (char* rules_file);
 void parse_metadata (TRACK_INFO* track_info);
