@@ -16,7 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,13 +35,15 @@
 #include "relaylib.h"
 #include "socklib.h"
 #include "external.h"
+#include "ripogg.h"
 
 /*****************************************************************************
  * Private functions
  *****************************************************************************/
 static error_code find_sep (u_long *pos1, u_long *pos2);
-static error_code end_track(u_long pos1, u_long pos2, TRACK_INFO* ti);
-static error_code start_track (TRACK_INFO* ti);
+static error_code end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti);
+static error_code start_track_mp3 (TRACK_INFO* ti);
+static error_code end_track_ogg (TRACK_INFO* ti);
 
 static void compute_cbuf2_size (SPLITPOINT_OPTIONS *sp_opt,
 				  int bitrate, int meta_interval);
@@ -51,9 +52,10 @@ static int bytes_to_secs (unsigned int bytes);
 static void clear_track_info (TRACK_INFO* ti);
 static int ripstream_recvall (char* buffer, int size);
 
-/* From ripshout */
 static error_code get_track_from_metadata (int size, char *newtrack);
-static error_code get_stream_data(char *data_buf, char *track_buf);
+static error_code get_stream_data (char *data_buf, char *track_buf);
+static error_code ripstream_rip_ogg (void);
+static error_code ripstream_rip_mp3 (void);
 
 /*****************************************************************************
  * Private Vars
@@ -89,7 +91,7 @@ static int m_have_relay;
 static int m_timeout;
 
 static int m_meta_interval;
-static int m_buffersize;
+static unsigned long m_buffersize;
 static int m_chunkcount;
 
 typedef struct ID3V1st
@@ -231,6 +233,7 @@ clear_track_info (TRACK_INFO* ti)
     ti->artist[0] = 0;
     ti->title[0] = 0;
     ti->album[0] = 0;
+    ti->track[0] = 0;
     ti->composed_metadata[0] = 0;
     ti->save_track = TRUE;
 }
@@ -249,7 +252,119 @@ copy_track_info (TRACK_INFO* dest, TRACK_INFO* src)
 
 /**** The main loop for ripping ****/
 error_code
-ripstream_rip()
+ripstream_rip (void)
+{
+    if (m_content_type == CONTENT_TYPE_OGG) {
+	return ripstream_rip_ogg ();
+    } else {
+	return ripstream_rip_mp3 ();
+    }
+}
+
+static error_code
+ripstream_rip_ogg (void)
+{
+    int ret;
+    int real_ret = SR_SUCCESS;
+    u_long extract_size;
+    static int have_track = 0;
+
+    /* get the data from the stream */
+    debug_printf ("RIPSTREAM_RIP_OGG: top of loop\n");
+    ret = get_stream_data(m_getbuffer, m_current_track.raw_metadata);
+    if (ret != SR_SUCCESS) {
+	debug_printf("get_stream_data bad return code: %d\n", ret);
+	return ret;
+    }
+
+    if (m_first_time_through) {
+	/* Allocate circular buffer */
+	m_bitrate = -1;
+	m_buffersize = 1024;
+	m_cbuf2_size = 128;
+	ret = cbuf2_init (&g_cbuf2, m_content_type, m_have_relay, 
+			  m_buffersize, m_cbuf2_size);
+	if (ret != SR_SUCCESS) return ret;
+	have_track = 0;
+	/* Warm up the ogg decoding routines */
+	rip_ogg_init ();
+	/* Done! */
+	m_first_time_through = 0;
+    }
+
+    /* Copy the data into cbuffer */
+    clear_track_info (&m_current_track);
+    ret = cbuf2_insert_chunk (&g_cbuf2, m_getbuffer, m_buffersize,
+			      m_content_type, &m_current_track);
+    if (ret != SR_SUCCESS) {
+	debug_printf("start_track had bad return code %d\n", ret);
+	return ret;
+    }
+
+    filelib_write_show (m_getbuffer, m_buffersize);
+
+    /* If we have unwritten pages for the current track, write them */
+    if (have_track) {
+	do {
+	    error_code ret;
+	    unsigned long amt_filled;
+	    int got_eos;
+	    ret = cbuf2_ogg_peek_song (&g_cbuf2, m_getbuffer, m_buffersize,
+				       &amt_filled, &got_eos);
+	    debug_printf ("^^^ogg_peek: %d %d\n", amt_filled, got_eos);
+	    if (ret != SR_SUCCESS) {
+		debug_printf ("cbuf2_ogg_peek_song: %d\n",ret);
+		return ret;
+	    }
+	    if (amt_filled == 0) {
+		break;
+	    }
+	    ret = rip_manager_put_data (m_getbuffer, amt_filled);
+	    if (ret != SR_SUCCESS) {
+		debug_printf ("rip_manager_put_data(#1): %d\n",ret);
+		return ret;
+	    }
+	    if (got_eos) {
+		end_track_ogg (&m_old_track);
+		have_track = 0;
+		break;
+	    }
+	} while (1);
+    }
+
+    format_track_info (&m_current_track, "current");
+
+    /* If we got a new track, then start a new file */
+    if (m_current_track.have_track_info) {
+	ret = rip_manager_start_track (&m_current_track, m_track_count);
+	if (ret != SR_SUCCESS) {
+	    debug_printf ("rip_manager_start_track failed(#1): %d\n",ret);
+	    return ret;
+	}
+	filelib_write_cue (&m_current_track, 0);
+	copy_track_info (&m_old_track, &m_current_track);
+
+	have_track = 1;
+    }
+
+    /* If buffer almost full, advance the buffer */
+    if (cbuf2_get_free(&g_cbuf2) < m_buffersize) {
+	debug_printf ("cbuf2_get_free < m_buffersize\n");
+	extract_size = m_buffersize - cbuf2_get_free(&g_cbuf2);
+
+        ret = cbuf2_advance_ogg (&g_cbuf2, m_buffersize);
+
+        if (ret != SR_SUCCESS) {
+	    debug_printf("cbuf2_extract had bad return code %d\n", ret);
+	    return ret;
+	}
+    }
+
+    return real_ret;
+}
+
+static error_code
+ripstream_rip_mp3 (void)
 {
     int ret;
     int real_ret = SR_SUCCESS;
@@ -267,7 +382,7 @@ ripstream_rip()
        The bitrate is needed to do the track splitting parameters 
        properly in seconds.  See the readme file for details.  */
     /* GCS FIX: For VBR streams, the header value may be more reliable. */
-    if (m_bitrate == -1) {
+    if (m_first_time_through) {
         unsigned long test_bitrate;
 	debug_printf("Querying stream for bitrate - first time.\n");
 	if (m_content_type == CONTENT_TYPE_MP3) {
@@ -287,7 +402,8 @@ ripstream_rip()
 		m_bitrate = 24;
 	}
         compute_cbuf2_size (m_sp_opt, m_bitrate, m_buffersize);
-	ret = cbuf2_init(&g_cbuf2, m_have_relay, m_buffersize, m_cbuf2_size);
+	ret = cbuf2_init (&g_cbuf2, m_content_type, m_have_relay, 
+			  m_buffersize, m_cbuf2_size);
 	if (ret != SR_SUCCESS) return ret;
     }
 
@@ -296,7 +412,6 @@ ripstream_rip()
 	clear_track_info (&m_current_track);
 	read_external (m_external_process, &m_current_track);
     } else {
-	/* Parse the metadata (RMK: ogg might have multiple metadata/chunk) */
 	if (m_current_track.raw_metadata[0]) {
 	    parse_metadata (&m_current_track);
 	} else {
@@ -305,10 +420,10 @@ ripstream_rip()
     }
 
     /* Copy the data into cbuffer */
-    ret = cbuf2_insert_chunk(&g_cbuf2, m_getbuffer, m_buffersize,
-			     &m_current_track);
+    ret = cbuf2_insert_chunk (&g_cbuf2, m_getbuffer, m_buffersize,
+			      m_content_type, &m_current_track);
     if (ret != SR_SUCCESS) {
-	debug_printf("start_track had bad return code %d\n", ret);
+	debug_printf("cbuf2_insert_chunk had bad return code %d\n", ret);
 	return ret;
     }
 
@@ -371,13 +486,13 @@ ripstream_rip()
 	}
 
 	/* Write out previous track */
-	ret = end_track(pos1, pos2, &m_old_track);
+	ret = end_track_mp3 (pos1, pos2, &m_old_track);
 	if (ret != SR_SUCCESS)
 	    real_ret = ret;
 	m_cue_sheet_bytes += pos2;
 
 	/* Start next track */
-	ret = start_track (&m_new_track);
+	ret = start_track_mp3 (&m_new_track);
 	if (ret != SR_SUCCESS)
 	    real_ret = ret;
 	m_find_silence = -1;
@@ -462,8 +577,8 @@ find_sep (u_long *pos1, u_long *pos2)
     return SR_SUCCESS;
 }
 
-error_code
-end_track(u_long pos1, u_long pos2, TRACK_INFO* ti)
+static error_code
+end_track_mp3 (u_long pos1, u_long pos2, TRACK_INFO* ti)
 {
     // pos1 is end of prev track
     // pos2 is beginning of next track
@@ -517,8 +632,8 @@ end_track(u_long pos1, u_long pos2, TRACK_INFO* ti)
     return ret;
 }
 
-error_code
-start_track (TRACK_INFO* ti)
+static error_code
+start_track_mp3 (TRACK_INFO* ti)
 {
 #define HEADER_SIZE 1600
     int ret;
@@ -636,6 +751,23 @@ start_track (TRACK_INFO* ti)
     return SR_SUCCESS;
 }
 
+// Only save this track if we've skipped over enough cruft 
+// at the beginning of the stream
+static error_code
+end_track_ogg (TRACK_INFO* ti)
+{
+    error_code ret;
+    debug_printf ("Current track number %d (skipping if %d or less)\n", 
+		  m_track_count, m_drop_count);
+    if (m_track_count > m_drop_count) {
+	ret = rip_manager_end_track (ti);
+    } else {
+	ret = SR_SUCCESS;
+    }
+    m_track_count ++;
+    return ret;
+}
+
 /* GCS: This converts either positive or negative ms to blocks,
    and must work for rounding up and rounding down */
 static int
