@@ -72,8 +72,8 @@ const char* codeset_relay;
 const char* codeset_id3;
 const char* codeset_filesys;
 
-
-char *subnstr_until(const char *str, char *until, char *newstr, int maxlen)
+char*
+subnstr_until(const char *str, char *until, char *newstr, int maxlen)
 {
     const char *p = str;
     int len = 0;
@@ -650,11 +650,13 @@ void trim(char *str)
    3) then add the null char
 */
 void
-sr_strncpy(char* dst, char* src, int n)
+sr_strncpy (char* dst, char* src, int n)
 {
     int i = 0;
     for (i = 0; i < n-1; i++) {
-	dst[i] = src[i];
+	if (!(dst[i] = src[i])) {
+	    return;
+	}
     }
     dst[i] = 0;
 }
